﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Text
Imports System.Linq
Imports System.Collections.Generic
Imports System
Imports System.Runtime.CompilerServices

Namespace IsCheckedBinding
    Public Class Club
        Implements INotifyPropertyChanged
        Private Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

        Private m_name As String
        Private m_established As DateTime
        Private m_stadiumCapacity As Integer
        Private m_players As ObservableCollection(Of Player)
        Private m_isChampion As Integer

        Public Property IsChampion() As Integer
            Get
                Return Me.m_isChampion
            End Get
            Set
                If Value <> Me.m_isChampion Then
                    Me.m_isChampion = Value
                    Me.OnPropertyChanged("IsChampion")
                End If
            End Set
        End Property

        Public Property Name() As String
            Get
                Return Me.m_name
            End Get
            Set
                If Value <> Me.m_name Then
                    Me.m_name = Value
                    Me.OnPropertyChanged("Name")
                End If
            End Set
        End Property

        Public Property Established() As DateTime
            Get
                Return Me.m_established
            End Get
            Set
                If Value <> Me.m_established Then
                    Me.m_established = Value
                    Me.OnPropertyChanged("Established")
                End If
            End Set
        End Property

        Public Property StadiumCapacity() As Integer
            Get
                Return Me.m_stadiumCapacity
            End Get
            Set
                If Value <> Me.m_stadiumCapacity Then
                    Me.m_stadiumCapacity = Value
                    Me.OnPropertyChanged("StadiumCapacity")
                End If
            End Set
        End Property

        Public ReadOnly Property Players() As ObservableCollection(Of Player)
            Get
                If Me.m_players Is Nothing Then
                    Me.m_players = New ObservableCollection(Of Player)()
                End If

                Return Me.m_players
            End Get
        End Property


        Public Sub New()
        End Sub

        Private Shared rand As New Random()

        Public Sub New(name As String, established As DateTime, stadiumCapacity As Integer)
            Me.m_name = name
            Me.m_established = established
            Me.m_stadiumCapacity = stadiumCapacity
            Me.m_isChampion = rand.Next(0, 2)
        End Sub

        Public Sub New(name As String, established As DateTime, stadiumCapacity As Integer, players As ObservableCollection(Of Player))
            Me.New(name, established, stadiumCapacity)
            Me.m_players = players
        End Sub

        Private Sub OnPropertyChanged(<CallerMemberName()> Optional ByVal propertyName As String = Nothing)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
        End Sub

        Public Overrides Function ToString() As String
            Return Me.Name
        End Function

        Public Shared Function GetClubs() As ObservableCollection(Of Club)
            Dim clubs As New ObservableCollection(Of Club)()
            Dim club As Club

            ' Liverpool
            club = New Club("Liverpool", New DateTime(1892, 1, 1), 45362)
            club.Players.Add(New Player("Pepe Reina", 25, Position.GK, "Spain"))
            club.Players.Add(New Player("Jamie Carragher", 23, Position.DF, "England"))
            club.Players.Add(New Player("Steven Gerrard", 8, Position.MF, "England"))
            club.Players.Add(New Player("Fernando Torres", 9, Position.FW, "Spain"))
            clubs.Add(club)

            ' Manchester Utd.
            club = New Club("Manchester Utd.", New DateTime(1878, 1, 1), 76212)
            club.Players.Add(New Player("Edwin van der Sar", 1, Position.GK, "Netherlands"))
            club.Players.Add(New Player("Rio Ferdinand", 5, Position.DF, "England"))
            club.Players.Add(New Player("Ryan Giggs", 11, Position.MF, "Wales"))
            club.Players.Add(New Player("Wayne Rooney", 10, Position.FW, "England"))
            clubs.Add(club)

            ' Chelsea
            club = New Club("Chelsea", New DateTime(1905, 1, 1), 42055)
            club.Players.Add(New Player("Petr Čech", 1, Position.GK, "Czech Republic"))
            club.Players.Add(New Player("John Terry", 26, Position.DF, "England"))
            club.Players.Add(New Player("Frank Lampard", 8, Position.MF, "England"))
            club.Players.Add(New Player("Nicolas Anelka", 39, Position.FW, "France"))
            clubs.Add(club)

            ' Arsenal
            club = New Club("Arsenal", New DateTime(1886, 1, 1), 60355)
            club.Players.Add(New Player("Manuel Almunia", 1, Position.GK, "Spain"))
            club.Players.Add(New Player("Gaël Clichy", 22, Position.DF, "France"))
            club.Players.Add(New Player("Cesc Fàbregas", 4, Position.MF, "Spain"))
            club.Players.Add(New Player("Robin van Persie", 11, Position.FW, "Netherlands"))
            clubs.Add(club)

            Return clubs
        End Function
    End Class
End Namespace